import time
from amoChatApi.TextMessage import TextMessage
from amoChatApi.PhotoMessage import PhotoMessage
class Payload:
    def __init__(self, api, type = "text"):
        self._json_payload = {"payload":{}}
        self.set_timestamp(int(time.time()))
        self.api = api
        self.set_silent(False)
        self.set_event_type("new_message")
        if type == "text":
            self.message:TextMessage = TextMessage()
        if type == "photo":
            self.message:PhotoMessage = PhotoMessage()
    def set_event_type(self, type:str):
        self._json_payload["event_type"] = type
    def set_timestamp(self, timestamp:int):
        self._json_payload["payload"]["timestamp"] = timestamp
        self.set_msec_timestamp(timestamp*1000)
    def set_msec_timestamp(self, msec_timestamp:int):
        self._json_payload["payload"]["msec_timestamp"] = msec_timestamp
    def set_sender_id(self, id:str):
        try:
            self._json_payload["payload"]["sender"]["id"] = id
        except:
            self._json_payload["payload"]["sender"] = {}
            self._json_payload["payload"]["sender"]["id"] = id
    def set_sender_name(self, name:str):
        try:
            self._json_payload["payload"]["sender"]["name"] = name
        except:
            self._json_payload["payload"]["sender"] = {}
            self._json_payload["payload"]["sender"]["name"] = name
    def set_conversation_id(self, id:str):
        self._json_payload["payload"]["conversation_id"] = id
    def set_silent(self, state:bool):
        self._json_payload["payload"]["silent"] = state
    def set_source_id(self, id:str):
        self._json_payload["payload"]["source"] = {"external_id": id}
    def set_message_id(self, id:str):
        self._json_payload["payload"]["msgid"] = id
    def send(self):
        self._json_payload["payload"]["message"] = self.message.get_json()
        r = self.api.send_or_edit_message(self._json_payload)
        return r



import hashlib
import hmac
import json
from email.utils import formatdate
from amoChatApi.Payload import Payload
import requests


class ChatApi:
    def __init__(self, secret: str, id,  account_id, title, source_id, hook_api_version = "v2"):
        self.source_id = source_id
        self._secret = secret
        self._account_id = account_id
        self._title = title
        self._hook_api_version = hook_api_version
        self._id = id
        self._path = "/v2/origin/custom/"
        self._url = "https://amojo.amocrm.ru"
        self._content_type = "application/json"
        r = self._connect()
        if r.status_code == 200:
            self._scope_id = json.loads(r.text)["scope_id"]
        else:
            raise ValueError("Cant connect to amoChatApi")
    def __del__(self):
        self._disconnect()
    def _connect(self):
        connect_path = self._path + self._id + "/connect"
        connect_url = self._url + connect_path
        body = {
            "account_id": self._account_id,
            "title": self._title,
            "hook_api_version": "v2"
        }
        request_body = json.dumps(body)
        checksum = hashlib.md5(request_body.encode("utf-8")).hexdigest()
        date = formatdate(timeval=None, localtime=False, usegmt=True)
        str_to_sign = "\n".join(["POST", checksum, self._content_type, date, connect_path])
        signature = hmac.new(self._secret.encode("utf-8"), str_to_sign.encode("utf-8"), hashlib.sha1).hexdigest()
        headers = {
            "Date": date,
            "Content-Type": self._content_type,
            'Content-MD5': checksum,
            'X-Signature': signature,
        }
        response = requests.post(connect_url, data=request_body, headers=headers)
        return response

    def _disconnect(self):
        disconnect_path = self._path + self._id + "/disconnect"
        disconnect_url = self._url + disconnect_path
        body = {
            "account_id": self._account_id
        }
        request_body = json.dumps(body)
        checksum = hashlib.md5(request_body.encode("utf-8")).hexdigest()
        date = formatdate(timeval=None, localtime=False, usegmt=True)
        str_to_sign = "\n".join(["DELETE", checksum, self._content_type, date, disconnect_path])
        signature = hmac.new(self._secret.encode("utf-8"), str_to_sign.encode("utf-8"), hashlib.sha1).hexdigest()
        headers = {
            "Date": date,
            "Content-Type": self._content_type,
            'Content-MD5': checksum,
            'X-Signature': signature,
        }
        response = requests.delete(disconnect_url, data=request_body, headers=headers)
        return response

    def create_chat(self, conversation_id, user):
        create_chat_path = self._path + self._scope_id + "/chats"
        create_chat_url = self._url + create_chat_path
        body = {
            "conversation_id": conversation_id,
            "user": user,
            "account_id": self._account_id,
            "source": {
                "external_id": self.source_id
            }
        }
        request_body = json.dumps(body)
        checksum = hashlib.md5(request_body.encode("utf-8")).hexdigest()
        date = formatdate(timeval=None, localtime=False, usegmt=True)
        str_to_sign = "\n".join(["POST", checksum, self._content_type, date, create_chat_path])
        signature = hmac.new(self._secret.encode("utf-8"), str_to_sign.encode("utf-8"), hashlib.sha1).hexdigest()
        headers = {
            "Date": date,
            "Content-Type": self._content_type,
            'Content-MD5': checksum,
            'X-Signature': signature,
        }
        response = requests.post(create_chat_url, data=request_body, headers=headers)
        return response

    def send_or_edit_message(self, payload):
        user = {
            "id": payload["payload"]["sender"]["id"],
            "name":payload["payload"]["sender"]["name"]
        }
        r = self.create_chat(payload["payload"]["conversation_id"], user)
        send_or_edit_message_path = self._path + self._scope_id
        send_or_edit_message_url = self._url + send_or_edit_message_path
        request_body = json.dumps(payload)
        checksum = hashlib.md5(request_body.encode("utf-8")).hexdigest()
        date = formatdate(timeval=None, localtime=False, usegmt=True)
        str_to_sign = "\n".join(["POST", checksum, self._content_type, date, send_or_edit_message_path])
        signature = hmac.new(self._secret.encode("utf-8"), str_to_sign.encode("utf-8"), hashlib.sha1).hexdigest()
        headers = {
            "Date": date,
            "Content-Type": self._content_type,
            'Content-MD5': checksum,
            'X-Signature': signature,
        }
        response = requests.post(send_or_edit_message_url, data=request_body, headers=headers)
        response.raise_for_status()
        return response

    def change_message_status(self, msgid:str, delivery_status:int, error_code:int, error:str):
        path = self._path + self._scope_id
        url = self._url + path
        body = {
            "msgid":msgid,
            "delivery_status":delivery_status,
            "error_code":error_code,
            "error":error
        }
        request_body = json.dumps(body)
        checksum = hashlib.md5(request_body.encode("utf-8")).hexdigest()
        date = formatdate(timeval=None, localtime=False, usegmt=True)
        str_to_sign = "\n".join(["POST", checksum, self._content_type, date, path])
        signature = hmac.new(self._secret.encode("utf-8"), str_to_sign.encode("utf-8"), hashlib.sha1).hexdigest()
        headers = {
            "Date": date,
            "Content-Type": self._content_type,
            'Content-MD5': checksum,
            'X-Signature': signature,
        }
        response = requests.post(url, data=request_body, headers=headers)
        response.raise_for_status()
        return response

    def create_new_text_message(self):
        return Payload(self)
    def create_new_photo_message(self):
        return Payload(self, "photo")




import json
from typing import List
class Lead:
    def __init__(self, api=None, json_lead:json = {}):
        self._json_lead = json_lead
        self.api = api

    def add_text_note(self, text: str):
        self.api.add_lead_text_note(lead=self, text=text)

    def from_json(self, json: json):
        self._json_lead = json

    def set_id(self, id: int):
        self._json_lead["id"] = id

    def set_name(self, name: str):
        self._json_lead["name"] = name

    def set_price(self, price: int):
        self._json_lead["price"] = price

    def set_status_id(self, status_id: int):
        self._json_lead["status_id"] = status_id

    def set_pipeline_id(self, pipeline_id: int):
        self._json_lead["pipeline_id"] = pipeline_id

    def set_created_by(self, created_by: int):
        self._json_lead["created_by"] = created_by

    def set_updated_by(self, updated_by: int):
        self._json_lead["updated_by"] = updated_by

    def set_closed_at(self, closed_at: int):
        self._json_lead["closed_at"] = closed_at

    def set_created_at(self, created_at: int):
        self._json_lead["created_at"] = created_at

    def set_updated_at(self, updated_at: int):
        self._json_lead["updated_at"] = updated_at

    def set_loss_reason_id(self, loss_reason_id: int):
        self._json_lead["loss_reason_id"] = loss_reason_id

    def set_responsible_user_id(self, responsible_user_id: int):
        self._json_lead["responsible_user_id"] = responsible_user_id
    def set_custom_field(self, id:int, values):
        if not self._json_lead.get("custom_fields_values"):
            self._json_lead["custom_fields_values"] = []
        field = {
            "field_id": id,
            "values": values
        }
        for i in range(0, len(self._json_lead["custom_fields_values"])):
            if self._json_lead["custom_fields_values"][i]["field_id"] == id:
                self._json_lead["custom_fields_values"][i]["values"] = values
                return
        self._json_lead["custom_fields_values"].append(field)

    # Getters
    def get_id(self) -> int:
        return self._json_lead.get("id")

    def get_name(self) -> str:
        return self._json_lead.get("name")

    def get_price(self) -> int:
        return self._json_lead.get("price")

    def get_status_id(self) -> int:
        return self._json_lead.get("status_id")

    def get_pipeline_id(self) -> int:
        return self._json_lead.get("pipeline_id")

    def get_created_by(self) -> int:
        return self._json_lead.get("created_by")

    def get_updated_by(self) -> int:
        return self._json_lead.get("updated_by")

    def get_closed_at(self) -> int:
        return self._json_lead.get("closed_at")

    def get_created_at(self) -> int:
        return self._json_lead.get("created_at")

    def get_updated_at(self) -> int:
        return self._json_lead.get("updated_at")

    def get_loss_reason_id(self) -> int:
        return self._json_lead.get("loss_reason_id")

    def get_responsible_user_id(self) -> int:
        return self._json_lead.get("responsible_user_id")

    def get_json(self) -> json:
        return self._json_lead
    def patch(self):
        self.api.patch_lead(self)



import requests
from amoApi.amoEntities.Lead import Lead
import json
from typing import List
from amoApi.amoEntities.Contact import Contact


class API:
    def __init__(self, token:str, url:str):
        self._auth(token, url)

    def _delete(self, method: str, headers=None, body=None) -> json:
        if body is None:
            body = {}
        if headers is None:
            headers = {}
        if not self._token:
            raise ValueError("Not authorized!")
        headers["Authorization"] = "Bearer " + self._token
        r = requests.delete("https://" + self._suburl + ".amocrm.ru/api/v4/" + method,
                          headers=headers, json=body)
        if r.status_code != 200:
            raise ValueError(r.status_code, r.text)
        return json.loads(r.text)
    def _get(self, method, headers=None, parameters=None) -> json:
        if headers is None:
            headers = {}
        if parameters is None:
            parameters = {}
        if not self._token:
            raise ValueError("Not authorized!")
        headers["Authorization"] = "Bearer " + self._token
        r = requests.get("https://" + self._suburl + ".amocrm.ru/api/v4/" + method,
                         headers=headers, params=parameters)
        if r.status_code != 200:
            if r.status_code == 204:
                raise ValueError("Not found!")
            raise ValueError(r.status_code, r.text)
        return json.loads(r.text)

    def _post(self, method: str, headers=None, body=None) -> json:
        if body is None:
            body = {}
        if headers is None:
            headers = {}
        if not self._token:
            raise ValueError("Not authorized!")
        headers["Authorization"] = "Bearer " + self._token
        r = requests.post("https://" + self._suburl + ".amocrm.ru/api/v4/" + method,
                          headers=headers, json=body)
        if r.status_code != 200:
            raise ValueError(r.status_code, r.text)
        return json.loads(r.text)

    def _patch(self, method, headers=None, body=None) -> json:
        if headers is None:
            headers = {}
        if body is None:
            body = {}
        if not self._token:
            raise ValueError("Not authorized!")
        headers["Authorization"] = "Bearer " + self._token
        r = requests.patch("https://" + self._suburl + ".amocrm.ru/api/v4/" + method,
                           headers=headers, json=body)
        if r.status_code != 200:
            raise ValueError(r.status_code, r.text)
        return json.loads(r.text)

    def _auth(self, token, url):
        headers = {"Authorization": "Bearer " + token}
        r = requests.get("https://" + url + ".amocrm.ru/api/v4/leads", headers=headers)
        if r.status_code == 200:
            self._token = token
            self._suburl = url
            self._url = "https://" + url + ".amocrm.ru/api/v4/"
        else:
            raise ValueError("Bad auth!")

    def get_lead(self, id: int) -> Lead:
        try:
            params = {}
            params["with"] = "contacts"
            r = self._get("leads/" + str(id), parameters=params)
            lead = Lead(self)
            lead.from_json(r)
            return lead
        except Exception as e:
            return None

    def get_leads(self, params: dict = {}) -> List[Lead]:
        leads = []
        response_leads = self._get("leads", parameters=params)
        if len(response_leads) == 0:
            return leads
        for lead in response_leads["_embedded"]["leads"]:
            l = Lead(self)
            l.from_json(lead)
            leads.append(l)
        return leads

    def complex_create_lead(self, body=None, contacts=None, fields=None) -> Lead:
        if body is None:
            body = [{}]
        if contacts is None:
            contacts = [{}]
        if fields is None:
            fields = [{}]
        body[0]["custom_fields_values"] = fields
        body[0]["_embedded"] = {}
        body[0]["_embedded"]["contacts"] = contacts
        headers = {"Authorization": "Bearer " + self._token}
        r = requests.post(self._url + "leads/complex", json=body, headers=headers)
        lead = Lead()
        lead.id = json.loads(r.text)[0]["id"]
        return lead
    def get_contact(self, id:int, params:dict = {}):
        params["with"] = "leads"
        contact = Contact(self, self._get("contacts/" + str(id), parameters=params))
        return contact
    def get_contact_links(self, contacts_id: List[int] = None, chats_id: List[str] = None):
        params = {}
        if not contacts_id is None:
            params["contact_id"] = contacts_id
        if not chats_id is None:
            params["chat_id"] = chats_id
        return self._get("contacts/chats", parameters=params)
    def get_lead_links(self, lead_id):
        return self._get("/api/v4/leads/" + str(lead_id) + "/links")
    def create_lead(self, params: dict = [{}]) -> Lead:
        r = self._post("leads", body=params)
        lead = Lead(self)
        lead.from_json(r["_embedded"]["leads"][0])
        return lead
    def patch_lead(self, lead:Lead):
        body = lead.get_json()
        try:
            for i in range(0, len(body["_embedded"]["tags"])):
                try:
                    body["_embedded"]["tags"][i].pop("color")
                except:
                    pass
        except:
            pass
        try:
            for i in range(0, len(body["_embedded"]["contacts"])):
                try:
                    body["_embedded"]["contacts"][i].pop("_links")
                except:
                    pass
        except:
            pass
        return self._patch("leads/"+str(lead.get_id()), body=lead.get_json())
    def add_lead_text_note(self, lead: Lead, text: str):
        body = [{}]
        body[0]["entity_id"] = lead.get_id()
        body[0]["note_type"] = "common"
        body[0]["params"] = {"text": text}
        r = self._post(method="leads/notes", body=body)

    def create_source(self, name:str, id:str):
        body = [{"name":name, "external_id": id}]
        return self._post("sources", body=body)
    def delete_source(self, id:int):
        body = [{"id":id}]
        return self._delete("sources", body=body)
    def get_sources(self):
        return self._get("sources")



from amoChatApi.AbstractMessage import Message
class PhotoMessage(Message):
    def __init__(self, photo_url:str = None, photo_name:str = None, file_size:int = None, text:str = ""):
        self._text = text
        self._photo_url = photo_url
        self._photo_name = photo_name
        self._file_size = file_size
    def get_json(self):
        json = {
            "type": "picture",
            "media": self._photo_url,
            "file_name": self._photo_name
        }
        if not self._text is None:
            json["text"] = self._text
        if not self._file_size is None:
            json["file_size"] = self._file_size
        return json
    def set_text(self, text:str):
        self._text = text
    def set_url(self, url:str):
        self._photo_url = url
    def set_name(self, name:str):
        self._photo_name = name
    def set_file_size(self, size:int):
        self._file_size = size
    def get_type(self):
        return "picture"